<?php

/**
 * P4A - PHP For Applications.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact the authors write to:									<br>
 * CreaLabs															<br>
 * Via Medail, 32													<br>
 * 10144 Torino (Italy)												<br>
 * Web:    {@link http://www.crealabs.it}							<br>
 * E-mail: {@link mailto:info@crealabs.it info@crealabs.it}
 *
 * The latest version of p4a can be obtained from:
 * {@link http://p4a.sourceforge.net}
 *
 * @link http://p4a.sourceforge.net
 * @link http://www.crealabs.it
 * @link mailto:info@crealabs.it info@crealabs.it
 * @copyright CreaLabs
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @author Fabrizio Balliano <fabrizio.balliano@crealabs.it>
 * @author Andrea Giardina <andrea.giardina@crealabs.it>
 * @package P4A_Base_Application
 */

class fatture extends P4A_Mask
{
	function fatture()
	{
		$this->P4A_Mask();
		$p4a =& P4A::singleton();
		
		// DB Source
		$this->build("p4a_db_source", "fatture");
		$this->fatture->setFields(array("fatture.*" => "*",
									   "Clienti.Ragione_sociale" => "cliente"
										));
    $this->fatture->setTable("fatture");
		// Add join to display customer's name instead of Id
    $this->fatture->addJoin("clienti","fatture.ID_cliente = clienti.ID");
		$this->fatture->setPk("ID");
		$this->fatture->addOrder("Num_fattura", P4A_ORDER_DESCENDING);
		$this->fatture->setPageLimit(5);
		$this->fatture->load();		
		$this->setSource($this->fatture);
		$this->firstRow();
		
 		$this->setTitle('Gestione fatture');

    //Create detail source. Detail table should be in a different class
    $dett_prestazioni= & $this->build("prestazioni", "dett_prestazioni");
    $dett_prestazioni->source->addFilter("ID_fattura = ?", 
                                          $this->fatture->fields->ID); 

		// Customizing fields properties
		$this->setFieldsProperties();
		$fields =& $this->fields;

		// Search Fieldset
		$fs_search =& $this->build("p4a_fieldset","fs_search");
		$fs_search->setTitle("Ricerca per numero fattura");
		$txt_search =& $this->build("p4a_field", "txt_search");
		$txt_search->addAction("onReturnPress");
		$this->intercept($txt_search, "onReturnPress","search");
		$txt_search->setLabel("Numero fattura");
		$cmd_search =& $this->build("p4a_button","cmd_search");
		$cmd_search->setValue("Cerca");
		$this->intercept($cmd_search, "onClick","search");
		$fs_search->anchor($txt_search);
		$fs_search->anchorLeft($cmd_search);
		$fs_search->setWidth(700);

    // Build Tab pane
    $tab_pane =& $this->build("p4a_tab_pane","tab_pane"); 

		// Toolbar
		$tool = & $this->build("p4a_simple_toolbar", "tool");
		// Hide original print screen button
    $tool->buttons->print->setInvisible();
    // Add new button to print pdf
    $tool->addButton('stampafattura','print');
    // Intercept onClick and override with our own method 
    $this->intercept($tool->buttons->stampafattura,'onClick','print_doc');
    // Join toolbar with this mask
    $tool->setMask($this);
		
		// Table
		$table_fatture =& $this->build("p4a_table", "table_fatture");
 		$table_fatture->setWidth(710);
		$table_fatture->setSource($this->fatture);
		$table_fatture->setVisibleCols(array("Num_fattura","Data_fattura","cliente",
                                         "Pagamento","Pagata"));
		$table_fatture->cols->Num_fattura->setLabel("N. fattura");
    // Display date with default locale (see index.php)
    $table_fatture->cols->Data_fattura->setFormat("datetime","date_default");  
		$table_fatture->showNavigationBar();

		// Message
		$message =& $this->build("p4a_message", "message");
		$message->setWidth("700");

		//Fieldset con l'elenco dei campi
		$fset=& $this->build("p4a_fieldset", "frame");
		$fset->setWidth(700);
		$fset->setTitle("Dati fattura");

		$fset->anchor($this->fields->Num_fattura);
		$fset->anchorLeft($this->fields->Data_fattura);
    $fset->anchor($this->fields->ID_cliente);
		$fset->anchorLeft($this->fields->Pagamento);
		$fset->anchor($this->fields->Pagata);

    // events onChange 
    // Intercept change page on the tab pane
    $this->intercept($tab_pane,"afterSetActivePage","ChangePage");

    // tabpane settings 
    $tab_pane->setWidth(800); 
    $tab1 =& $tab_pane->pages->build("p4a_frame","tab1"); 
    $tab2 =& $tab_pane->pages->build("p4a_frame","tab2"); 
    $tab1->setLabel("Fatture");
    $tab2->setLabel("Dettaglio fattura");

		// Frame
		$frm=& $this->build("p4a_frame", "frm");
		$frm->setWidth(800);
		
    // anchor tab pane
    $frm->anchor($tab_pane); 
		// tab1 - Anchor master table
    $tab1->anchor($fs_search);
    $tab1->newRow();    
    $tab1->anchorCenter($message);
    $tab1->anchor($table_fatture); 
    $tab1->anchor($fset); 
    // tab2 - Anchor detail table
    $tab2->anchor($dett_prestazioni->table); 
    $tab2->anchor($dett_prestazioni->toolbar);
    $tab2->anchor($dett_prestazioni->frm);  


		// Mandatory Fields
	   $this->mf = array("Num_fattura","Data_fattura");
		foreach($this->mf as $mf){
			$fields->$mf->label->setFontWeight("bold");
		}

		// Display
		$this->display("main", $frm);
		$this->display("menu", $p4a->menu);
		$this->display("top", $this->tool);		
	}

	function setFieldsProperties()
	{
		$p4a =& p4a::singleton();

		$fields =& $this->fields;
    $fields->Num_fattura->setLabel("N. fattura");
 		$fields->Data_fattura->setType("date");
    $fields->Data_fattura->setFormat("datetime","date_default");
    // Build source for a lookup customers combo
 		$clienti =& $this->build("P4A_DB_Source","clienti");
		$clienti->setTable("clienti");
		$clienti->setPK("ID");
		$clienti->addOrder("Ragione_sociale");
		$clienti->setFields(array("clienti.*" => "*", 
                        "clienti.Ragione_sociale" => "identificativocliente"));
   	$clienti->load();
		// Build combo
    $fields->ID_cliente->setType("select");
		$fields->ID_cliente->setSource($clienti);
		$fields->ID_cliente->setSourceDescriptionField("identificativocliente");
		$fields->ID_cliente->setLabel("Cliente");  
		
	}

	function saveRow()
	{
		$valid = true;

		foreach($this->mf as $mf){
			$value = $this->fields->$mf->getNewValue();
			if(trim($value) === ""){
				$this->fields->$mf->setError();
				$valid = false;
			}
		}

		if ($valid) {
      // Reset date format for inserting/updating records
      $this->fields->Data_fattura->unsetFormat();
      parent::saveRow();
		}else{
			$this->message->setValue("Indicare tutti i campi necessari");
		}
	}

	function search()
	{
    $value = $this->txt_search->getSQLNewValue();
    if(!empty($value)) {
		  $this->data->setWhere("Num_fattura = '{$value}'");
		}
		else {
			$this->data->setWhere(null);
    }
		$this->data->firstRow();
		$num_rows = $this->data->getNumRows();
		if (!$num_rows) {
			$this->message->setValue("Nessuna fattura trovata");
		}
  }
	

  function ChangePage() 
  { 
    $newpage = $this->tab_pane->getActivePageName(); 
    switch ($newpage) 
    { 
      case "tab1":         
        // If display master, enable all toolbar buttons
        $this->tool->buttons->new->enable();
        $this->tool->buttons->save->enable();
        $this->tool->buttons->delete->enable();
        $this->tool->buttons->print->enable();        
        //  force to redesign toolbar
        $this->tool->redesign();        
        break;  
      case "tab2":
        // If display detail, disable some master table buttons 
        $this->tool->buttons->new->disable();
        $this->tool->buttons->save->disable();
        $this->tool->buttons->delete->disable();
        $this->tool->buttons->print->disable();
        //  force to redesign toolbar
        $this->tool->redesign();
        // Go to first row and fill fieldset
        $this->dett_prestazioni->firstRow();
        break;  
    } 
  }
	
  function print_doc()
  // Function to print selected invoice with ezPdf class 
  { 
      require('class.ezpdf.php');      
      $pdf =& new Cezpdf('a4','portrait');
      // Select font family
      $pdf->selectFont($_SERVER['DOCUMENT_ROOT'].P4A_APPLICATION_PATH.
                       '/libraries/fonts/Helvetica.afm');      
      // Define some usefull styling tags (like html)
      $tmp = array(
        'b'=>'Helvetica-Bold.afm'
        ,'i'=>'Helvetica-Oblique.afm'
        ,'bi'=>'Helvetica-BoldOblique.afm'
        ,'ib'=>'Helvetica-BoldOblique.afm'
        ,'bb'=>'Helvetica-Bold.afm'
      );
      $pdf->setFontFamily("Helvetica.afm",$tmp);
      
      $db =& p4a_db::singleton();

      // Get from db header information
      $headerpdf=$db->getAll("SELECT Ragione_sociale, Indirizzo, 
                              CONCAT('Tel: ',Telefono), 
                              CONCAT('Fax: ',Fax), 
                              CONCAT('Email: ',email), 
                              CONCAT('C.F.: ',CF), 
                              CONCAT('P.IVA: ',PIVA), 
                              CONCAT('Banca: ',Dati_bancari) 
                              FROM intestazione");
      // Define xy init set up values
      $x=65;
      $y=820;
      $pdf->ezSetY($y);
      // Cycle through fields and print header
       foreach($headerpdf[0] as $key => $value){
        $pdf->ezSetY($y);
        $pdf->ezText($value,10,array('aleft'=>$x,'justification'=>'left'));
        $y-=10;
       }
      // Retrieve actual customer's id
      $actualIdCliente = $this->fatture->fields->ID_cliente->getNewValue();
      // Get customer's data from db       
      $invoiceData = $db->getAll("SELECT Ragione_sociale, indirizzo, 
                          CONCAT_WS(' - ', CAP, Citta, Provincia),  
                          PIVA, CF 
                          FROM clienti 
                          WHERE ID = '$actualIdCliente'"); 

      $y-=20;
      $pdf->ezSetY($y);
      $pdf->ezText("Spett.le ",10,array('aleft'=>'300','justification'=>'left'));
			
      $x=350;
      // Cycle through fields and print data            
       foreach($invoiceData[0] as $key => $value){
        $pdf->ezSetY($y);
        $pdf->ezText($value,10,array('aleft'=>$x,'justification'=>'left'));
        $y-=10;
       }
      // Print invoice number and date (we have it!)
      $numDateInvoice = "Fattura n. <b>".$this->fields->Num_fattura->getValue().
                        "</b> del <b><i>".
                        $this->fields->Data_fattura->getNewValue()."</i></b>";
      $x=65;
      $y-=20;
      $pdf->ezSetY($y);                        
      $pdf->ezText($numDateInvoice,10,array('aleft'=>$x,'justification'=>'left'));

       // Retrieve actual invoice id      
      $actualIdFattura = $this->fatture->fields->ID->getNewValue();      
       // Get invoice detail items from db
      $detInvoiceData = $db->getAll("SELECT Descrizione, Importo, 
                                    CONCAT(Aliquota_IVA,'%') 
                                    FROM prestazioni 
                                    WHERE ID_Fattura = '$actualIdFattura'");
       // Print header fields name
      $y-=30;
      $pdf->ezSetY($y);
      $pdf->ezText("Descrizione",12,array('aleft'=>'65','justification'=>'left'));
      $pdf->ezSetY($y);
      $pdf->ezText("Importo",12,array('aleft'=>'415', 'aright'=>'485',
                    'justification'=>'right'));
      $pdf->ezSetY($y);
      $pdf->ezText("IVA",12,array('aleft'=>'515','justification'=>'left'));

      $y-=15;
      $pdf->line(65,$y,540,$y);

			$x=65;
      $y-=10;
      // Cycle through records and fields and print data      
      foreach($detInvoiceData as $row){
        foreach($row as $key => $value){
          $pdf->ezSetY($y);
          if($key=="Importo"){
            //format price with 2 decimals
            $formattedValue = sprintf("%01.2f", $value);
            $pdf->ezText($formattedValue,10,array('aleft'=>$x,'aright'=>$x+70,
                         'justification'=>'right'));
          }
          else{
            $pdf->ezText($value,10,array('aleft'=>$x,'justification'=>'left'));            
          }
          if($x==65)
            $x+=350;
          else
          $x+=100;         
        }
        $x=65;
        $y-=10;
      }
      // Get total amount price and tax with an aggregation query
      $totPriceInvoice = $db->getAll( "SELECT fatture.ID, 
      SUM( prestazioni.importo) AS TotaleImponibile,
      SUM( prestazioni.importo * (prestazioni.Aliquota_IVA /100)) AS TotaleIVA, 
      SUM(prestazioni.importo * (1 + (prestazioni.Aliquota_IVA /100))) AS Totale
      FROM fatture 
      RIGHT JOIN prestazioni ON fatture.ID = prestazioni.ID_fattura
      GROUP BY fatture.ID HAVING fatture.ID='$actualIdFattura'");
      
      $y-=25;
      $pdf->line(65,$y,540,$y);
      // Print a summary with total price without tax, total tax,
      // and total price including tax.
      $y-=50;
      $pdf->ezSetY($y);
      $pdf->ezText('<b>Totale imponibile</b>',10,array('aleft'=>'300',
                   'justification'=>'left'));
      $pdf->ezSetY($y);
      $formattedValue = sprintf("%01.2f",
                                $totPriceInvoice[0]['TotaleImponibile']);
      $pdf->ezText("<b>".$formattedValue."</b>",10,
                    array('aleft'=>'415','aright'=>'485','justification'=>'right'));      
      $y-=12;
      $pdf->ezSetY($y);
      $pdf->ezText('<b>Totale IVA</b>',10,array('aleft'=>'300',
                   'justification'=>'left'));
      $pdf->ezSetY($y);
      $formattedValue = sprintf("%01.2f",
                                $totPriceInvoice[0]['TotaleIVA']);      
      $pdf->ezText("<b>".$formattedValue."</b>",10,
                    array('aleft'=>'415','aright'=>'485','justification'=>'right'));
      $y-=12;
      $pdf->ezSetY($y);
      $pdf->ezText('<b>Totale fattura</b>',10,array('aleft'=>'300',
                   'justification'=>'left'));
      $pdf->ezSetY($y);
      $formattedValue = sprintf("%01.2f",
                                $totPriceInvoice[0]['Totale']);      
      $pdf->ezText("<b>".$formattedValue."</b>",10,
                    array('aleft'=>'415','aright'=>'485','justification'=>'right'));      

      // Set up header pdf infos
      $type = 'application/pdf'; 
      $filename = 'fattura.pdf';        
      header("Pragma: public"); 
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0"); 
      header("Cache-Control: private", false); 
      header("Content-type: $type"); 
      // forcing browser to recognize pdf as attachment, will open new window
      header("Content-Disposition: attachment; filename=\"$filename\""); 
      $pdfcode = $pdf->output(1); // data stream without header
      header("Content-Length: " . strlen($pdfcode)); 
      echo $pdfcode; 
      // done! now we must kill new process!
      die();
  } 
	
	function main() 
	//this method runs every mask reload
	{
    // Redesign fieldset with appropriate format
    $this->setFieldsProperties();
		parent::main(); //show mask 
	}

	
}

?>
